@@ -8,9 +8,6 @@
 
 import json
 import sys
-import urllib.request
-import re
-from os import environ
 
 if len(sys.argv) == 2:
     build_type = sys.argv[1]
@@ -32,43 +29,28 @@ def clean_os(os):
 def clean_ssl(ssl):
     return ssl.replace("_VERSION", "").lower()
 
-def determine_latest_openssl(ssl):
-    openssl_tags = urllib.request.urlopen("https://api.github.com/repos/openssl/openssl/tags")
-    tags = json.loads(openssl_tags.read().decode('utf-8'))
-    latest_tag = ''
-    for tag in tags:
-        name = tag['name']
-        if "openssl-" in name:
-            if name > latest_tag:
-               latest_tag = name
-    return "OPENSSL_VERSION={}".format(latest_tag[8:])
-
-def determine_latest_libressl(ssl):
-    libressl_download_list = urllib.request.urlopen("http://ftp.openbsd.org/pub/OpenBSD/LibreSSL/")
-    for line in libressl_download_list.readlines():
-        decoded_line = line.decode("utf-8")
-        if "libressl-" in decoded_line and ".tar.gz.asc" in decoded_line:
-             l = re.split("libressl-|.tar.gz.asc", decoded_line)[1]
-    return "LIBRESSL_VERSION={}".format(l)
 
 def clean_compression(compression):
     return compression.replace("USE_", "").lower()
 
 
 def get_asan_flags(cc):
-    return [
-        "USE_OBSOLETE_LINKER=1",
-        'DEBUG_CFLAGS="-g -fsanitize=address"',
-        'LDFLAGS="-fsanitize=address"',
-        'CPU_CFLAGS.generic="-O1"',
-    ]
+    if cc == "clang":
+        return [
+            "USE_OBSOLETE_LINKER=1",
+            'DEBUG_CFLAGS="-g -fsanitize=address"',
+            'LDFLAGS="-fsanitize=address"',
+            'CPU_CFLAGS.generic="-O1"',
+        ]
+
+    raise ValueError("ASAN is only supported for clang")
 
 
 matrix = []
 
 # Ubuntu
 
-os = "ubuntu-20.04"
+os = "ubuntu-latest"
 TARGET = "linux-glibc"
 for CC in ["gcc", "clang"]:
     matrix.append(
@@ -123,7 +105,8 @@ for CC in ["gcc", "clang"]:
         "stock",
         "OPENSSL_VERSION=1.0.2u",
         "OPENSSL_VERSION=3.0.2",
-        "LIBRESSL_VERSION=3.5.3",
+        "LIBRESSL_VERSION=2.9.2",
+        "LIBRESSL_VERSION=3.5.1",
 #        "BORINGSSL=yes",
     ]:
         flags = ["USE_OPENSSL=1"]
@@ -132,11 +115,6 @@ for CC in ["gcc", "clang"]:
         if ssl != "stock":
             flags.append("SSL_LIB=${HOME}/opt/lib")
             flags.append("SSL_INC=${HOME}/opt/include")
-        if "LIBRESSL" in ssl and "latest" in ssl:
-            ssl = determine_latest_libressl(ssl)
-        if "OPENSSL" in ssl and "latest" in ssl:
-            ssl = determine_latest_openssl(ssl)
-
         matrix.append(
             {
                 "name": "{}, {}, ssl={}".format(clean_os(os), CC, clean_ssl(ssl)),
@@ -150,34 +128,34 @@ for CC in ["gcc", "clang"]:
 
 # ASAN
 
-os = "ubuntu-20.04"
+os = "ubuntu-latest"
+CC = "clang"
 TARGET = "linux-glibc"
-for CC in ["gcc","clang"]:
-    matrix.append(
-	{
-            "name": "{}, {}, ASAN, all features".format(clean_os(os), CC),
-            "os": os,
-            "TARGET": TARGET,
-            "CC": CC,
-            "FLAGS": get_asan_flags(CC)
-            + [
-		"USE_ZLIB=1",
-		"USE_PCRE=1",
-		"USE_PCRE_JIT=1",
-		"USE_LUA=1",
-		"USE_OPENSSL=1",
-		"USE_SYSTEMD=1",
-		"USE_WURFL=1",
-		"WURFL_INC=contrib/wurfl",
-		"WURFL_LIB=contrib/wurfl",
-		"USE_DEVICEATLAS=1",
-		"DEVICEATLAS_SRC=contrib/deviceatlas",
-		"EXTRA_OBJS=contrib/prometheus-exporter/service-prometheus.o",
-		"USE_51DEGREES=1",
-		"51DEGREES_SRC=contrib/51d/src/pattern",
-            ],
-	}
-    )
+matrix.append(
+    {
+        "name": "{}, {}, ASAN, all features".format(clean_os(os), CC),
+        "os": os,
+        "TARGET": TARGET,
+        "CC": CC,
+        "FLAGS": get_asan_flags(CC)
+        + [
+            "USE_ZLIB=1",
+            "USE_PCRE=1",
+            "USE_PCRE_JIT=1",
+            "USE_LUA=1",
+            "USE_OPENSSL=1",
+            "USE_SYSTEMD=1",
+            "USE_WURFL=1",
+            "WURFL_INC=contrib/wurfl",
+            "WURFL_LIB=contrib/wurfl",
+            "USE_DEVICEATLAS=1",
+            "DEVICEATLAS_SRC=contrib/deviceatlas",
+            "EXTRA_OBJS=contrib/prometheus-exporter/service-prometheus.o",
+            "USE_51DEGREES=1",
+            "51DEGREES_SRC=contrib/51d/src/pattern",
+        ],
+    }
+)
 
 # macOS
 
@@ -198,6 +176,4 @@ for CC in ["clang"]:
 
 print(json.dumps(matrix, indent=4, sort_keys=True))
 
-if environ.get('GITHUB_OUTPUT') is not None:
-    with open(environ.get('GITHUB_OUTPUT'), 'a') as f:
-        print("matrix={}".format(json.dumps({"include": matrix})), file=f)
+print("::set-output name=matrix::{}".format(json.dumps({"include": matrix})))
