@@ -67,8 +67,8 @@ enum {
 
 
 	CS_FL_ERROR         = 0x00000100,  /* a fatal error was reported */
-	CS_FL_RCV_MORE      = 0x00000200,  /* We may have more bytes to transfert */
-	CS_FL_WANT_ROOM     = 0x00000400,  /* More bytes to transfert, but not enough room */
+	CS_FL_RCV_MORE      = 0x00000200,  /* We may have more bytes to transfer */
+	CS_FL_WANT_ROOM     = 0x00000400,  /* More bytes to transfer, but not enough room */
 	CS_FL_ERR_PENDING   = 0x00000800,  /* An error is pending, but there's still data to be read */
 	CS_FL_EOS           = 0x00001000,  /* End of stream delivered to data layer */
 	/* unused: 0x00002000 */
@@ -127,7 +127,7 @@ enum {
 	CO_FL_CTRL_READY    = 0x00000100, /* FD was registered, fd_delete() needed */
 	CO_FL_XPRT_READY    = 0x00000200, /* xprt_init() done, xprt_close() needed */
 
-	/* unused : 0x00000400 */
+	CO_FL_WANT_DRAIN    = 0x00000400, /* try to drain pending data when closing */
 
 	/* This flag is used by data layers to indicate they had to stop
 	 * receiving data because a buffer was full. The connection handler
@@ -243,6 +243,16 @@ enum {
 	CO_ER_SOCKS4_ABORT,      /* SOCKS4 Proxy handshake aborted by server */
 };
 
+/* error return codes for accept_conn() */
+enum {
+	CO_AC_NONE = 0,  /* no error, valid connection returned */
+	CO_AC_DONE,      /* reached the end of the queue (typically EAGAIN) */
+	CO_AC_RETRY,     /* late signal delivery or anything requiring the caller to try again */
+	CO_AC_YIELD,     /* short-lived limitation that requires a short pause */
+	CO_AC_PAUSE,     /* long-lived issue (resource/memory allocation error, paused FD) */
+	CO_AC_PERMERR,   /* permanent, non-recoverable error (e.g. closed listener socket) */
+};
+
 /* source address settings for outgoing connections */
 enum {
 	/* Tproxy exclusive values from 0 to 7 */
@@ -282,6 +292,7 @@ enum {
 	MX_FL_NONE        = 0x00000000,
 	MX_FL_CLEAN_ABRT  = 0x00000001, /* abort is clearly reported as an error */
 	MX_FL_HTX         = 0x00000002, /* set if it is an HTX multiplexer */
+	MX_FL_HOL_RISK    = 0x00000004, /* set if the protocol is subject the to head-of-line blocking on server */
 };
 
 /* PROTO token registration */
@@ -356,6 +367,8 @@ struct xprt_ops {
 	void (*destroy_srv)(struct server *srv);    /* destroy a server context */
 	int  (*get_alpn)(const struct connection *conn, void *xprt_ctx, const char **str, int *len); /* get application layer name */
 	int (*takeover)(struct connection *conn, void *xprt_ctx, int orig_tid); /* Let the xprt know the fd have been taken over */
+	void (*set_idle)(struct connection *conn, void *xprt_ctx); /* notify the xprt that the connection becomes idle. implies set_used. */
+	void (*set_used)(struct connection *conn, void *xprt_ctx); /* notify the xprt that the connection leaves idle. implies set_idle. */
 	char name[8];                               /* transport layer name, zero-terminated */
 	int (*subscribe)(struct connection *conn, void *xprt_ctx, int event_type, struct wait_event *es); /* Subscribe <es> to events, such as "being able to send" */
 	int (*unsubscribe)(struct connection *conn, void *xprt_ctx, int event_type, struct wait_event *es); /* Unsubscribe <es> from events */
@@ -491,7 +504,6 @@ struct connection {
 	struct sockaddr_storage *src; /* source address (pool), when known, otherwise NULL */
 	struct sockaddr_storage *dst; /* destination address (pool), when known, otherwise NULL */
 	char *proxy_authority;	      /* Value of authority TLV received via PROXYv2 */
-	unsigned int idle_time;                 /* Time the connection was added to the idle list, or 0 if not in the idle list */
 	uint8_t proxy_authority_len;  /* Length of authority TLV received via PROXYv2 */
 	struct ist proxy_unique_id;  /* Value of the unique ID TLV received via PROXYv2 */
 };
@@ -609,8 +621,8 @@ struct tlv_ssl {
  */
 struct idle_conns {
 	struct mt_list toremove_conns;
-	__decl_thread(HA_SPINLOCK_T takeover_lock);
 	struct task *cleanup_task;
+	__decl_thread(HA_SPINLOCK_T takeover_lock);
 } THREAD_ALIGNED(64);
 
 #endif /* _HAPROXY_CONNECTION_T_H */
