@@ -144,11 +144,15 @@ struct dns_answer_item {
 	int16_t         priority;                    /* SRV type priority */
 	uint16_t        weight;                      /* SRV type weight */
 	uint16_t        port;                        /* SRV type port */
-	uint16_t        data_len;                    /* number of bytes in target below */
-	struct sockaddr address;                     /* IPv4 or IPv6, network format */
-	char            target[DNS_MAX_NAME_SIZE+1]; /* Response data: SRV or CNAME type target */
+	uint16_t        data_len;                    /* number of bytes in the <data> field below */
+	union {
+		struct sockaddr_in in4;              /* IPv4 address for RTYPE_A */
+		struct sockaddr_in6 in6;             /* IPv6 address for RTYPE_AAAA */
+		char target[DNS_MAX_NAME_SIZE+1];    /* Response data: SRV or CNAME type target */
+	} data;
 	unsigned int    last_seen;                   /* When was the answer was last seen */
 	struct dns_answer_item *ar_item;             /* pointer to a RRset from the additional section, if exists */
+	struct list     attached_servers;            /* attached server head */
 	struct list     list;
 };
 
@@ -356,6 +360,8 @@ struct dns_srvrq {
 	char                 *hostname_dn;      /* server hostname in Domain Name format */
 	int                   hostname_dn_len;  /* string length of the server hostname in Domain Name format */
 	struct dns_requester *dns_requester;    /* used to link to its DNS resolution */
+	struct list attached_servers;           /* List of the servers free to use */
+	struct eb_root named_servers;           /* tree of servers indexed by hostnames found in server state file */
 	struct list list;                       /* Next SRV RQ for the same proxy */
 };
 
