@@ -89,6 +89,7 @@
 #define SF_IGNORE_PRST	0x00080000	/* ignore persistence */
 
 #define SF_SRV_REUSED   0x00100000	/* the server-side connection was reused */
+#define SF_SRV_REUSED_ANTICIPATED  0x00200000  /* the connection was reused but the mux is not ready yet */
 
 
 /* flags for the proxy of the master CLI */
@@ -134,7 +135,7 @@ struct stream {
 	struct http_txn *txn;           /* current HTTP transaction being processed. Should become a list. */
 
 	struct task *task;              /* the task associated with this stream */
-	unsigned short pending_events;	/* the pending events not yet processed by the stream.
+	unsigned int pending_events;	/* the pending events not yet processed by the stream.
 					 * This is a bit field of TASK_WOKEN_* */
 	int16_t priority_class;         /* priority class of the stream for the pending queue */
 	int32_t priority_offset;        /* priority offset of the stream for the pending queue */
@@ -144,7 +145,7 @@ struct stream {
 	struct list back_refs;          /* list of users tracking this stream */
 	struct buffer_wait buffer_wait; /* position in the list of objects waiting for a buffer */
 
-	struct freq_ctr call_rate;      /* stream task call rate */
+	struct freq_ctr call_rate;      /* stream task call rate without making progress */
 
 	short store_count;
 	enum obj_type obj_type;         /* object type == OBJ_TYPE_STREAM */
