@@ -101,7 +101,6 @@ struct acl_keyword *find_acl_kw(const char *kw)
 static struct acl_expr *prune_acl_expr(struct acl_expr *expr)
 {
 	struct arg *arg;
-	int unresolved = 0;
 
 	pattern_prune(&expr->pat);
 
@@ -110,13 +109,12 @@ static struct acl_expr *prune_acl_expr(struct acl_expr *expr)
 			break;
 		if (arg->type == ARGT_STR || arg->unresolved) {
 			chunk_destroy(&arg->data.str);
-			unresolved |= arg->unresolved;
 			arg->unresolved = 0;
 		}
 	}
 
-	if (expr->smp->arg_p != empty_arg_list && !unresolved)
-		free(expr->smp->arg_p);
+	release_sample_expr(expr->smp);
+
 	return expr;
 }
 
