@@ -22,6 +22,28 @@
 #include <haproxy/tools.h>
 
 
+/* Check an action ruleset validity. It returns the number of error encountered
+ * andd err_code is updated if a warning is emitted.
+ */
+int check_action_rules(struct list *rules, struct proxy *px, int *err_code)
+{
+	struct act_rule *rule;
+	char *errmsg = NULL;
+	int err = 0;
+
+	list_for_each_entry(rule, rules, list) {
+		if (rule->check_ptr && !rule->check_ptr(rule, px, &errmsg)) {
+			ha_alert("Proxy '%s': %s.\n", px->id, errmsg);
+			err++;
+		}
+
+		free(errmsg);
+		errmsg = NULL;
+	}
+
+	return err;
+}
+
 /* Find and check the target table used by an action track-sc*. This
  * function should be called during the configuration validity check.
  *
@@ -106,7 +128,7 @@ int check_trk_action(struct act_rule *rule, struct proxy *px, char **err)
 int check_capture(struct act_rule *rule, struct proxy *px, char **err)
 {
 	if (rule->from == ACT_F_TCP_REQ_CNT && (px->cap & PR_CAP_FE) && !px->tcp_req.inspect_delay &&
-	    !(rule->arg.trk_ctr.expr->fetch->val & SMP_VAL_FE_SES_ACC)) {
+	    !(rule->arg.cap.expr->fetch->val & SMP_VAL_FE_SES_ACC)) {
 		ha_warning("config : %s '%s' : a 'tcp-request capture' rule explicitly depending on request"
 			   " contents without any 'tcp-request inspect-delay' setting."
 			   " This means that this rule will randomly find its contents. This can be fixed by"
@@ -133,6 +155,12 @@ int act_resolution_cb(struct dns_requester *requester, struct dns_nameserver *na
 	return 0;
 }
 
+/*
+ * Do resolve error management callback
+ * returns:
+ *  0 if we can trash answser items.
+ *  1 when safely ignored and we must kept answer items
+ */
 int act_resolution_error_cb(struct dns_requester *requester, int error_code)
 {
 	struct stream *stream;
