@@ -651,7 +651,7 @@ int assign_server(struct stream *s)
 		/* if there's some queue on the backend, with certain algos we
 		 * know it's because all servers are full.
 		 */
-		if (s->be->nbpend &&
+		if (s->be->nbpend && s->be->nbpend != s->be->beconn &&
 		    (((s->be->lbprm.algo & (BE_LB_KIND|BE_LB_NEED|BE_LB_PARM)) == BE_LB_ALGO_FAS)||   // first
 		     ((s->be->lbprm.algo & (BE_LB_KIND|BE_LB_NEED|BE_LB_PARM)) == BE_LB_ALGO_RR) ||   // roundrobin
 		     ((s->be->lbprm.algo & (BE_LB_KIND|BE_LB_NEED|BE_LB_PARM)) == BE_LB_ALGO_SRR))) { // static-rr
@@ -819,7 +819,7 @@ out_ok:
 			sess_change_server(s, srv);
 		} else {
 			if (may_dequeue_tasks(conn_slot, s->be))
-				process_srv_queue(conn_slot);
+				process_srv_queue(conn_slot, 0);
 		}
 	}
 
@@ -1492,11 +1492,9 @@ skip_reuse:
 			srv_conn->mux->reset(srv_conn);
 	}
 	else {
-		/* Only consider we're doing reuse if the connection was
-		 * ready.
-		 */
-		if (srv_conn->mux->ctl(srv_conn, MUX_STATUS, NULL) & MUX_STATUS_READY)
-			s->flags |= SF_SRV_REUSED;
+		s->flags |= SF_SRV_REUSED;
+		if (!(srv_conn->mux->ctl(srv_conn, MUX_STATUS, NULL) & MUX_STATUS_READY))
+			s->flags |= SF_SRV_REUSED_ANTICIPATED;
 	}
 
 	/* flag for logging source ip/port */
@@ -1522,7 +1520,7 @@ skip_reuse:
 		return err;
 
 #ifdef USE_OPENSSL
-	if (srv && srv->ssl_ctx.sni) {
+	if (!(s->flags & SF_SRV_REUSED) && srv && srv->ssl_ctx.sni) {
 		struct sample *smp;
 
 		smp = sample_fetch_as_type(s->be, s->sess, s, SMP_OPT_DIR_REQ | SMP_OPT_FINAL,
@@ -1734,7 +1732,7 @@ int srv_redispatch_connect(struct stream *s)
 
 		/* release other streams waiting for this server */
 		if (may_dequeue_tasks(srv, s->be))
-			process_srv_queue(srv);
+			process_srv_queue(srv, 0);
 		return 1;
 	}
 	/* if we get here, it's because we got SRV_STATUS_OK, which also
@@ -1810,7 +1808,7 @@ void back_try_conn_req(struct stream *s)
 			/* release other streams waiting for this server */
 			sess_change_server(s, NULL);
 			if (may_dequeue_tasks(srv, s->be))
-				process_srv_queue(srv);
+				process_srv_queue(srv, 0);
 
 			/* Failed and not retryable. */
 			si_shutr(si);
@@ -2096,7 +2094,7 @@ void back_handle_st_cer(struct stream *s)
 
 	/* we probably have to release last stream from the server */
 	if (objt_server(s->target)) {
-		health_adjust(objt_server(s->target), HANA_STATUS_L4_ERR);
+		health_adjust(__objt_server(s->target), HANA_STATUS_L4_ERR);
 
 		if (s->flags & SF_CURR_SESS) {
 			s->flags &= ~SF_CURR_SESS;
@@ -2138,7 +2136,7 @@ void back_handle_st_cer(struct stream *s)
 		_HA_ATOMIC_ADD(&s->be->be_counters.failed_conns, 1);
 		sess_change_server(s, NULL);
 		if (may_dequeue_tasks(objt_server(s->target), s->be))
-			process_srv_queue(objt_server(s->target));
+			process_srv_queue(objt_server(s->target), 0);
 
 		/* shutw is enough so stop a connecting socket */
 		si_shutw(si);
@@ -2153,6 +2151,18 @@ void back_handle_st_cer(struct stream *s)
 		goto end;
 	}
 
+	/* At this stage, we will trigger a connection retry (with or without
+	 * redispatch). Thus we must release the SI endpoint on the server side
+	 * an close the attached connection. It is especially important to do it
+	 * now if the retry is not immediately performed, to be sure to release
+	 * ressources as soon as possible and to not catch errors from the lower
+	 * layers in an unexpected state (i.e < ST_CONN).
+	 *
+	 * Note: the stream-interface will be switched to ST_REQ, ST_ASS or
+	 * ST_TAR and SI_FL_ERR and SI_FL_EXP flags will be unset.
+	 */
+	si_release_endpoint(&s->si[1]);
+
 	stream_choose_redispatch(s);
 
 	if (si->flags & SI_FL_ERR) {
@@ -2165,6 +2175,8 @@ void back_handle_st_cer(struct stream *s)
 		 */
 
 		int delay = 1000;
+		const int reused = (s->flags & SF_SRV_REUSED) &&
+		                   !(s->flags & SF_SRV_REUSED_ANTICIPATED);
 
 		if (s->be->timeout.connect && s->be->timeout.connect < delay)
 			delay = s->be->timeout.connect;
@@ -2175,7 +2187,7 @@ void back_handle_st_cer(struct stream *s)
 		/* only wait when we're retrying on the same server */
 		if ((si->state == SI_ST_ASS ||
 		     (s->be->lbprm.algo & BE_LB_KIND) != BE_LB_KIND_RR ||
-		     (s->be->srv_act <= 1)) && !(s->flags & SF_SRV_REUSED)) {
+		     (s->be->srv_act <= 1)) && !reused) {
 			si->state = SI_ST_TAR;
 			si->exp = tick_add(now_ms, MS_TO_TICKS(delay));
 		}
