@@ -977,7 +977,7 @@ static void http_cache_io_handler(struct appctx *appctx)
 	unsigned int len;
 	size_t ret, total = 0;
 
-	res_htx = htxbuf(&res->buf);
+	res_htx = htx_from_buf(&res->buf);
 	total = res_htx->data;
 
 	if (unlikely(si->state == SI_ST_DIS || si->state == SI_ST_CLO))
@@ -1043,6 +1043,7 @@ static void http_cache_io_handler(struct appctx *appctx)
 			si_rx_room_blk(si);
 			goto out;
 		}
+		res->flags |= CF_EOI;
 		appctx->st0 = HTX_CACHE_END;
 	}
 
@@ -1343,10 +1344,11 @@ enum act_return http_action_req_cache_use(struct act_rule *rule, struct proxy *p
 				_HA_ATOMIC_ADD(&px->be_counters.p.http.cache_hits, 1);
 			return ACT_RET_CONT;
 		} else {
+			s->target = NULL;
 			shctx_lock(shctx_ptr(cache));
 			shctx_row_dec_hot(shctx_ptr(cache), block_ptr(res));
 			shctx_unlock(shctx_ptr(cache));
-			return ACT_RET_YIELD;
+			return ACT_RET_CONT;
 		}
 	}
 	shctx_unlock(shctx_ptr(cache));
@@ -1722,9 +1724,19 @@ static int cli_io_handler_show_cache(struct appctx *appctx)
 			}
 
 			entry = container_of(node, struct cache_entry, eb);
-			chunk_printf(&trash, "%p hash:%u size:%u (%u blocks), refcount:%u, expire:%d\n", entry, read_u32(entry->hash), block_ptr(entry)->len, block_ptr(entry)->block_count, block_ptr(entry)->refcount, entry->expire - (int)now.tv_sec);
-
 			next_key = node->key + 1;
+
+			if (entry->expire > now.tv_sec) {
+				chunk_printf(&trash, "%p hash:%u size:%u (%u blocks), refcount:%u, expire:%d\n",
+					     entry, read_u32(entry->hash),
+					     block_ptr(entry)->len, block_ptr(entry)->block_count,
+					     block_ptr(entry)->refcount, entry->expire - (int)now.tv_sec);
+			} else {
+				/* time to remove that one */
+				eb32_delete(&entry->eb);
+				entry->eb.key = 0;
+			}
+
 			appctx->ctx.cli.i0 = next_key;
 
 			shctx_unlock(shctx_ptr(cache));
