@@ -266,21 +266,21 @@ int prepare_external_check(struct check *check)
 
 	list_for_each_entry(l, &px->conf.listeners, by_fe)
 		/* Use the first INET, INET6 or UNIX listener */
-		if (l->addr.ss_family == AF_INET ||
-		    l->addr.ss_family == AF_INET6 ||
-		    l->addr.ss_family == AF_UNIX) {
+		if (l->rx.addr.ss_family == AF_INET ||
+		    l->rx.addr.ss_family == AF_INET6 ||
+		    l->rx.addr.ss_family == AF_UNIX) {
 			listener = l;
 			break;
 		}
 
 	check->curpid = NULL;
-	check->envp = calloc((EXTCHK_SIZE + 1), sizeof(char *));
+	check->envp = calloc((EXTCHK_SIZE + 1), sizeof(*check->envp));
 	if (!check->envp) {
 		ha_alert("Failed to allocate memory for environment variables. Aborting\n");
 		goto err;
 	}
 
-	check->argv = calloc(6, sizeof(char *));
+	check->argv = calloc(6, sizeof(*check->argv));
 	if (!check->argv) {
 		ha_alert("Starting [%s:%s] check: out of memory.\n", px->id, s->id);
 		goto err;
@@ -292,17 +292,17 @@ int prepare_external_check(struct check *check)
 		check->argv[1] = strdup("NOT_USED");
 		check->argv[2] = strdup("NOT_USED");
 	}
-	else if (listener->addr.ss_family == AF_INET ||
-	    listener->addr.ss_family == AF_INET6) {
-		addr_to_str(&listener->addr, buf, sizeof(buf));
+	else if (listener->rx.addr.ss_family == AF_INET ||
+	    listener->rx.addr.ss_family == AF_INET6) {
+		addr_to_str(&listener->rx.addr, buf, sizeof(buf));
 		check->argv[1] = strdup(buf);
-		port_to_str(&listener->addr, buf, sizeof(buf));
+		port_to_str(&listener->rx.addr, buf, sizeof(buf));
 		check->argv[2] = strdup(buf);
 	}
-	else if (listener->addr.ss_family == AF_UNIX) {
+	else if (listener->rx.addr.ss_family == AF_UNIX) {
 		const struct sockaddr_un *un;
 
-		un = (struct sockaddr_un *)&listener->addr;
+		un = (struct sockaddr_un *)&listener->rx.addr;
 		check->argv[1] = strdup(un->sun_path);
 		check->argv[2] = strdup("NOT_USED");
 	}
@@ -477,7 +477,7 @@ out:
  * Please do NOT place any return statement in this function and only leave
  * via the out_unlock label.
  */
-struct task *process_chk_proc(struct task *t, void *context, unsigned short state)
+struct task *process_chk_proc(struct task *t, void *context, unsigned int state)
 {
 	struct check *check = context;
 	struct server *s = check->server;
@@ -496,7 +496,7 @@ struct task *process_chk_proc(struct task *t, void *context, unsigned short stat
 		 * is disabled.
 		 */
 		if (((check->state & (CHK_ST_ENABLED | CHK_ST_PAUSED)) != CHK_ST_ENABLED) ||
-		    s->proxy->state == PR_STSTOPPED)
+		    s->proxy->disabled)
 			goto reschedule;
 
 		/* we'll initiate a new check */
