@@ -113,6 +113,7 @@ THREAD_LOCAL int poller_rd_pipe = -1; // Pipe to wake the thread
 int poller_wr_pipe[MAX_THREADS]; // Pipe to wake the threads
 
 volatile int ha_used_fds = 0; // Number of FD we're currently using
+static struct fdtab *fdtab_addr;  /* address of the allocated area containing fdtab */
 
 #define _GET_NEXT(fd, off) ((volatile struct fdlist_entry *)(void *)((char *)(&fdtab[fd]) + off))->next
 #define _GET_PREV(fd, off) ((volatile struct fdlist_entry *)(void *)((char *)(&fdtab[fd]) + off))->prev
@@ -461,7 +462,6 @@ __decl_spinlock(log_lock);
 ssize_t fd_write_frag_line(int fd, size_t maxlen, const struct ist pfx[], size_t npfx, const struct ist msg[], size_t nmsg, int nl)
 {
 	struct iovec iovec[32];
-	size_t totlen = 0;
 	size_t sent = 0;
 	int vec = 0;
 	int attempts = 0;
@@ -488,7 +488,6 @@ ssize_t fd_write_frag_line(int fd, size_t maxlen, const struct ist pfx[], size_t
 		iovec[vec].iov_base = pfx->ptr;
 		iovec[vec].iov_len  = MIN(maxlen, pfx->len);
 		maxlen -= iovec[vec].iov_len;
-		totlen += iovec[vec].iov_len;
 		if (iovec[vec].iov_len)
 			vec++;
 		pfx++; npfx--;
@@ -689,11 +688,14 @@ int init_pollers()
 	int p;
 	struct poller *bp;
 
-	if ((fdtab = calloc(global.maxsock, sizeof(*fdtab))) == NULL) {
+	if ((fdtab_addr = calloc(global.maxsock, sizeof(*fdtab) + 64)) == NULL) {
 		ha_alert("Not enough memory to allocate %d entries for fdtab!\n", global.maxsock);
 		goto fail_tab;
 	}
 
+	/* always provide an aligned fdtab */
+	fdtab = (struct fdtab*)((((size_t)fdtab_addr) + 63) & -(size_t)64);
+
 	if ((polled_mask = calloc(global.maxsock, sizeof(*polled_mask))) == NULL) {
 		ha_alert("Not enough memory to allocate %d entries for polled_mask!\n", global.maxsock);
 		goto fail_polledmask;
@@ -730,7 +732,7 @@ int init_pollers()
  fail_info:
 	free(polled_mask);
  fail_polledmask:
-	free(fdtab);
+	free(fdtab_addr);
  fail_tab:
 	return 0;
 }
@@ -751,7 +753,7 @@ void deinit_pollers() {
 	}
 
 	free(fdinfo);   fdinfo   = NULL;
-	free(fdtab);    fdtab    = NULL;
+	free(fdtab_addr);    fdtab_addr    = NULL;
 	free(polled_mask); polled_mask = NULL;
 }
 
