@@ -25,6 +25,8 @@
 #include <haproxy/stream.h>
 #include <haproxy/tools.h>
 
+#define COMP_STATE_PROCESSING 0x01
+
 const char *http_comp_flt_id = "compression filter";
 
 struct flt_ops comp_ops;
@@ -32,6 +34,7 @@ struct flt_ops comp_ops;
 struct comp_state {
 	struct comp_ctx  *comp_ctx;   /* compression context */
 	struct comp_algo *comp_algo;  /* compression algorithm if not NULL */
+	unsigned int      flags;      /* COMP_STATE_* */
 };
 
 /* Pools used to allocate comp_state structs */
@@ -93,6 +96,7 @@ comp_strm_init(struct stream *s, struct filter *filter)
 
 	st->comp_algo = NULL;
 	st->comp_ctx  = NULL;
+	st->flags     = 0;
 	filter->ctx   = st;
 
 	/* Register post-analyzer on AN_RES_WAIT_HTTP because we need to
@@ -135,6 +139,7 @@ comp_http_headers(struct stream *s, struct filter *filter, struct http_msg *msg)
 			if (!set_compression_response_header(st, s, msg))
 				goto end;
 			register_data_filter(s, msg->chn, filter);
+			st->flags |= COMP_STATE_PROCESSING;
 		}
 	}
 
@@ -213,7 +218,7 @@ comp_http_payload(struct stream *s, struct filter *filter, struct http_msg *msg,
 			case HTX_BLK_TLR:
 			case HTX_BLK_EOT:
 			case HTX_BLK_EOM:
-				if (msg->flags & HTTP_MSGF_COMPRESSING) {
+				if (st->flags & COMP_STATE_PROCESSING) {
 					if (htx_compression_buffer_init(htx, &trash) < 0) {
 						msg->chn->flags |= CF_WAKE_WRITE;
 						goto end;
@@ -229,9 +234,9 @@ comp_http_payload(struct stream *s, struct filter *filter, struct http_msg *msg,
 							goto error;
 						to_forward += b_data(&trash);
 					}
-					msg->flags &= ~HTTP_MSGF_COMPRESSING;
 					/* We let the mux add last empty chunk and empty trailers */
 				}
+				st->flags &= ~COMP_STATE_PROCESSING;
 				/* fall through */
 
 			default:
@@ -291,6 +296,7 @@ static int
 set_compression_response_header(struct comp_state *st, struct stream *s, struct http_msg *msg)
 {
 	struct htx *htx = htxbuf(&msg->chn->buf);
+	struct htx_sl *sl;
 	struct http_hdr_ctx ctx;
 
 	/*
@@ -306,17 +312,25 @@ set_compression_response_header(struct comp_state *st, struct stream *s, struct
 			goto error;
 	}
 
+	sl = http_get_stline(htx);
+	if (!sl)
+		goto error;
+
 	/* remove Content-Length header */
 	if (msg->flags & HTTP_MSGF_CNT_LEN) {
 		ctx.blk = NULL;
 		while (http_find_header(htx, ist("Content-Length"), &ctx, 1))
 			http_remove_header(htx, &ctx);
+		msg->flags &= ~HTTP_MSGF_CNT_LEN;
+		sl->flags &= ~HTX_SL_F_CLEN;
 	}
 
 	/* add "Transfer-Encoding: chunked" header */
 	if (!(msg->flags & HTTP_MSGF_TE_CHNK)) {
 		if (!http_add_header(htx, ist("Transfer-Encoding"), ist("chunked")))
 			goto error;
+		msg->flags |= HTTP_MSGF_TE_CHNK;
+		sl->flags |= (HTX_SL_F_XFER_ENC|HTX_SL_F_CHNK);
 	}
 
 	/* convert "ETag" header to a weak ETag */
@@ -639,11 +653,17 @@ parse_compression_options(char **args, int section, struct proxy *proxy,
 			return -1;
 		}
 		while (*(args[cur_arg])) {
-			if (comp_append_algo(comp, args[cur_arg]) < 0) {
-				memprintf(err, "'%s' : '%s' is not a supported algorithm.\n",
-					  args[0], args[cur_arg]);
+			int retval = comp_append_algo(comp, args[cur_arg]);
+			if (retval) {
+				if (retval < 0)
+					memprintf(err, "'%s' : '%s' is not a supported algorithm.\n",
+						  args[0], args[cur_arg]);
+				else
+					memprintf(err, "'%s' : out of memory while parsing algo '%s'.\n",
+						  args[0], args[cur_arg]);
 				return -1;
 			}
+
 			if (proxy->comp->algos->init(&ctx, 9) == 0)
 				proxy->comp->algos->end(&ctx);
 			else {
@@ -665,7 +685,10 @@ parse_compression_options(char **args, int section, struct proxy *proxy,
 			return -1;
 		}
 		while (*(args[cur_arg])) {
-			comp_append_type(comp, args[cur_arg]);
+			if (comp_append_type(comp, args[cur_arg])) {
+				memprintf(err, "'%s': out of memory.", args[0]);
+				return -1;
+			}
 			cur_arg++;
 			continue;
 		}
