@@ -10,7 +10,6 @@
  *
  */
 
-#define _GNU_SOURCE
 #include <ctype.h>
 #include <errno.h>
 #include <stdio.h>
@@ -29,7 +28,6 @@
 #include <haproxy/list.h>
 #include <haproxy/listener.h>
 #include <haproxy/log.h>
-#include <haproxy/proto_sockpair.h>
 #include <haproxy/protocol-t.h>
 #include <haproxy/protocol.h>
 #include <haproxy/sample.h>
@@ -44,18 +42,10 @@ static struct bind_kw_list bind_keywords = {
 	.list = LIST_HEAD_INIT(bind_keywords.list)
 };
 
-struct xfer_sock_list *xfer_sock_list = NULL;
-
-/* there is one listener queue per thread so that a thread unblocking the
- * global queue can wake up listeners bound only to foreign threads by
- * moving them to the remote queues and waking up the associated tasklet.
- */
-static struct work_list *local_listener_queue;
-
 /* list of the temporarily limited listeners because of lack of resource */
 static struct mt_list global_listener_queue = MT_LIST_HEAD_INIT(global_listener_queue);
 static struct task *global_listener_queue_task;
-static struct task *manage_global_listener_queue(struct task *t, void *context, unsigned short state);
+static struct task *manage_global_listener_queue(struct task *t, void *context, unsigned int state);
 
 
 #if defined(USE_THREAD)
@@ -63,22 +53,18 @@ static struct task *manage_global_listener_queue(struct task *t, void *context,
 struct accept_queue_ring accept_queue_rings[MAX_THREADS] __attribute__((aligned(64))) = { };
 
 /* dequeue and process a pending connection from the local accept queue (single
- * consumer). Returns the accepted fd or -1 if none was found. The listener is
- * placed into *li. The address is copied into *addr for no more than *addr_len
- * bytes, and the address length is returned into *addr_len.
+ * consumer). Returns the accepted connection or NULL if none was found.
  */
-int accept_queue_pop_sc(struct accept_queue_ring *ring, struct listener **li, void *addr, int *addr_len)
+struct connection *accept_queue_pop_sc(struct accept_queue_ring *ring)
 {
-	struct accept_queue_entry *e;
 	unsigned int pos, next;
-	struct listener *ptr;
-	int len;
-	int fd;
+	struct connection *ptr;
+	struct connection **e;
 
 	pos = ring->head;
 
 	if (pos == ring->tail)
-		return -1;
+		return NULL;
 
 	next = pos + 1;
 	if (next >= ACCEPT_QUEUE_SIZE)
@@ -88,42 +74,28 @@ int accept_queue_pop_sc(struct accept_queue_ring *ring, struct listener **li, vo
 
 	/* wait for the producer to update the listener's pointer */
 	while (1) {
-		ptr = e->listener;
+		ptr = *e;
 		__ha_barrier_load();
 		if (ptr)
 			break;
 		pl_cpu_relax();
 	}
 
-	fd = e->fd;
-	len = e->addr_len;
-	if (len > *addr_len)
-		len = *addr_len;
-
-	if (likely(len > 0))
-		memcpy(addr, &e->addr, len);
-
 	/* release the entry */
-	e->listener = NULL;
+	*e = NULL;
 
 	__ha_barrier_store();
 	ring->head = next;
-
-	*addr_len = len;
-	*li = ptr;
-
-	return fd;
+	return ptr;
 }
 
 
-/* tries to push a new accepted connection <fd> into ring <ring> for listener
- * <li>, from address <addr> whose length is <addr_len>. Returns non-zero if it
- * succeeds, or zero if the ring is full. Supports multiple producers.
+/* tries to push a new accepted connection <conn> into ring <ring>. Returns
+ * non-zero if it succeeds, or zero if the ring is full. Supports multiple
+ * producers.
  */
-int accept_queue_push_mp(struct accept_queue_ring *ring, int fd,
-                         struct listener *li, const void *addr, int addr_len)
+int accept_queue_push_mp(struct accept_queue_ring *ring, struct connection *conn)
 {
-	struct accept_queue_entry *e;
 	unsigned int pos, next;
 
 	pos = ring->tail;
@@ -135,48 +107,32 @@ int accept_queue_push_mp(struct accept_queue_ring *ring, int fd,
 			return 0; // ring full
 	} while (unlikely(!_HA_ATOMIC_CAS(&ring->tail, &pos, next)));
 
-
-	e = &ring->entry[pos];
-
-	if (addr_len > sizeof(e->addr))
-		addr_len = sizeof(e->addr);
-
-	if (addr_len)
-		memcpy(&e->addr, addr, addr_len);
-
-	e->addr_len = addr_len;
-	e->fd = fd;
-
+	ring->entry[pos] = conn;
 	__ha_barrier_store();
-	/* now commit the change */
-
-	e->listener = li;
 	return 1;
 }
 
 /* proceed with accepting new connections */
-static struct task *accept_queue_process(struct task *t, void *context, unsigned short state)
+static struct task *accept_queue_process(struct task *t, void *context, unsigned int state)
 {
 	struct accept_queue_ring *ring = context;
+	struct connection *conn;
 	struct listener *li;
-	struct sockaddr_storage addr;
 	unsigned int max_accept;
-	int addr_len;
 	int ret;
-	int fd;
 
 	/* if global.tune.maxaccept is -1, then max_accept is UINT_MAX. It
 	 * is not really illimited, but it is probably enough.
 	 */
-	max_accept = global.tune.maxaccept ? global.tune.maxaccept : 64;
+	max_accept = global.tune.maxaccept ? global.tune.maxaccept : MAX_ACCEPT;
 	for (; max_accept; max_accept--) {
-		addr_len = sizeof(addr);
-		fd = accept_queue_pop_sc(ring, &li, &addr, &addr_len);
-		if (fd < 0)
+		conn = accept_queue_pop_sc(ring);
+		if (!conn)
 			break;
 
+		li = __objt_listener(conn->target);
 		_HA_ATOMIC_ADD(&li->thr_conn[tid], 1);
-		ret = li->accept(li, fd, &addr);
+		ret = li->accept(conn);
 		if (ret <= 0) {
 			/* connection was terminated by the application */
 			continue;
@@ -226,62 +182,221 @@ REGISTER_CONFIG_POSTPARSER("multi-threaded accept queue", accept_queue_init);
 
 #endif // USE_THREAD
 
+/* adjust the listener's state and its proxy's listener counters if needed.
+ * It must be called under the listener's lock, but uses atomic ops to change
+ * the proxy's counters so that the proxy lock is not needed.
+ */
+void listener_set_state(struct listener *l, enum li_state st)
+{
+	struct proxy *px = l->bind_conf->frontend;
+
+	if (px) {
+		/* from state */
+		switch (l->state) {
+		case LI_NEW: /* first call */
+			_HA_ATOMIC_ADD(&px->li_all, 1);
+			break;
+		case LI_INIT:
+		case LI_ASSIGNED:
+			break;
+		case LI_PAUSED:
+			_HA_ATOMIC_SUB(&px->li_paused, 1);
+			break;
+		case LI_LISTEN:
+			_HA_ATOMIC_SUB(&px->li_bound, 1);
+			break;
+		case LI_READY:
+		case LI_FULL:
+		case LI_LIMITED:
+			_HA_ATOMIC_SUB(&px->li_ready, 1);
+			break;
+		}
+
+		/* to state */
+		switch (st) {
+		case LI_NEW:
+		case LI_INIT:
+		case LI_ASSIGNED:
+			break;
+		case LI_PAUSED:
+			BUG_ON(l->rx.fd == -1);
+			_HA_ATOMIC_ADD(&px->li_paused, 1);
+			break;
+		case LI_LISTEN:
+			BUG_ON(l->rx.fd == -1);
+			_HA_ATOMIC_ADD(&px->li_bound, 1);
+			break;
+		case LI_READY:
+		case LI_FULL:
+		case LI_LIMITED:
+			BUG_ON(l->rx.fd == -1);
+			_HA_ATOMIC_ADD(&px->li_ready, 1);
+			break;
+		}
+	}
+	l->state = st;
+}
+
 /* This function adds the specified listener's file descriptor to the polling
  * lists if it is in the LI_LISTEN state. The listener enters LI_READY or
  * LI_FULL state depending on its number of connections. In daemon mode, we
  * also support binding only the relevant processes to their respective
  * listeners. We don't do that in debug mode however.
  */
-static void enable_listener(struct listener *listener)
+void enable_listener(struct listener *listener)
 {
-	HA_SPIN_LOCK(LISTENER_LOCK, &listener->lock);
+	HA_RWLOCK_WRLOCK(LISTENER_LOCK, &listener->lock);
+
+	/* If this listener is supposed to be only in the master, close it in
+	 * the workers. Conversely, if it's supposed to be only in the workers
+	 * close it in the master.
+	 */
+	if (!!master != !!(listener->rx.flags & RX_F_MWORKER))
+		do_unbind_listener(listener);
+
 	if (listener->state == LI_LISTEN) {
+		BUG_ON(listener->rx.fd == -1);
 		if ((global.mode & (MODE_DAEMON | MODE_MWORKER)) &&
-		    !(proc_mask(listener->bind_conf->bind_proc) & pid_bit)) {
+		    (!!master != !!(listener->rx.flags & RX_F_MWORKER) ||
+		     !(proc_mask(listener->rx.settings->bind_proc) & pid_bit))) {
 			/* we don't want to enable this listener and don't
 			 * want any fd event to reach it.
 			 */
-			if (!(global.tune.options & GTUNE_SOCKET_TRANSFER))
-				do_unbind_listener(listener, 1);
-			else {
-				do_unbind_listener(listener, 0);
-				listener->state = LI_LISTEN;
-			}
+			do_unbind_listener(listener);
 		}
 		else if (!listener->maxconn || listener->nbconn < listener->maxconn) {
-			fd_want_recv(listener->fd);
-			listener->state = LI_READY;
+			listener->rx.proto->enable(listener);
+			listener_set_state(listener, LI_READY);
 		}
 		else {
-			listener->state = LI_FULL;
+			listener_set_state(listener, LI_FULL);
 		}
 	}
-	/* if this listener is supposed to be only in the master, close it in the workers */
-	if ((global.mode & MODE_MWORKER) &&
-	    (listener->options & LI_O_MWORKER) &&
-	    master == 0) {
-		do_unbind_listener(listener, 1);
-	}
-	HA_SPIN_UNLOCK(LISTENER_LOCK, &listener->lock);
+
+	HA_RWLOCK_WRUNLOCK(LISTENER_LOCK, &listener->lock);
 }
 
-/* This function removes the specified listener's file descriptor from the
- * polling lists if it is in the LI_READY or in the LI_FULL state. The listener
- * enters LI_LISTEN.
+/*
+ * This function completely stops a listener. It will need to operate under the
+ * proxy's lock, the protocol's lock, and the listener's lock. The caller is
+ * responsible for indicating in lpx, lpr, lli whether the respective locks are
+ * already held (non-zero) or not (zero) so that the function picks the missing
+ * ones, in this order. The proxy's listeners count is updated and the proxy is
+ * disabled and woken up after the last one is gone.
  */
-static void disable_listener(struct listener *listener)
+void stop_listener(struct listener *l, int lpx, int lpr, int lli)
 {
-	HA_SPIN_LOCK(LISTENER_LOCK, &listener->lock);
-	if (listener->state < LI_READY)
-		goto end;
-	if (listener->state == LI_READY)
-		fd_stop_recv(listener->fd);
-	MT_LIST_DEL(&listener->wait_queue);
-	listener->state = LI_LISTEN;
-  end:
-	HA_SPIN_UNLOCK(LISTENER_LOCK, &listener->lock);
+	struct proxy *px = l->bind_conf->frontend;
+
+	if (l->options & LI_O_NOSTOP) {
+		/* master-worker sockpairs are never closed but don't count as a
+		 * job.
+		 */
+		return;
+	}
+
+	if (!lpx)
+		HA_RWLOCK_WRLOCK(PROXY_LOCK, &px->lock);
+
+	if (!lpr)
+		HA_SPIN_LOCK(PROTO_LOCK, &proto_lock);
+
+	if (!lli)
+		HA_RWLOCK_WRLOCK(LISTENER_LOCK, &l->lock);
+
+	if (l->state > LI_INIT) {
+		do_unbind_listener(l);
+
+		if (l->state >= LI_ASSIGNED)
+			__delete_listener(l);
+
+		proxy_cond_disable(px);
+	}
+
+	if (!lli)
+		HA_RWLOCK_WRUNLOCK(LISTENER_LOCK, &l->lock);
+
+	if (!lpr)
+		HA_SPIN_UNLOCK(PROTO_LOCK, &proto_lock);
+
+	if (!lpx)
+		HA_RWLOCK_WRUNLOCK(PROXY_LOCK, &px->lock);
 }
 
+/* This function adds the specified <listener> to the protocol <proto>. It
+ * does nothing if the protocol was already added. The listener's state is
+ * automatically updated from LI_INIT to LI_ASSIGNED. The number of listeners
+ * for the protocol is updated. This must be called with the proto lock held.
+ */
+void default_add_listener(struct protocol *proto, struct listener *listener)
+{
+	if (listener->state != LI_INIT)
+		return;
+	listener_set_state(listener, LI_ASSIGNED);
+	listener->rx.proto = proto;
+	LIST_ADDQ(&proto->receivers, &listener->rx.proto_list);
+	proto->nb_receivers++;
+}
+
+/* default function called to suspend a listener: it simply passes the call to
+ * the underlying receiver. This is find for most socket-based protocols. This
+ * must be called under the listener's lock. It will return non-zero on success,
+ * 0 on failure. If no receiver-level suspend is provided, the operation is
+ * assumed to succeed.
+ */
+int default_suspend_listener(struct listener *l)
+{
+	int ret = 1;
+
+	if (!l->rx.proto->rx_suspend)
+		return 1;
+
+	ret = l->rx.proto->rx_suspend(&l->rx);
+	return ret > 0 ? ret : 0;
+}
+
+
+/* Tries to resume a suspended listener, and returns non-zero on success or
+ * zero on failure. On certain errors, an alert or a warning might be displayed.
+ * It must be called with the listener's lock held. Depending on the listener's
+ * state and protocol, a listen() call might be used to resume operations, or a
+ * call to the receiver's resume() function might be used as well. This is
+ * suitable as a default function for TCP and UDP. This must be called with the
+ * listener's lock held.
+ */
+int default_resume_listener(struct listener *l)
+{
+	int ret = 1;
+
+	if (l->state == LI_ASSIGNED) {
+		char msg[100];
+		int err;
+
+		err = l->rx.proto->listen(l, msg, sizeof(msg));
+		if (err & ERR_ALERT)
+			ha_alert("Resuming listener: %s\n", msg);
+		else if (err & ERR_WARN)
+			ha_warning("Resuming listener: %s\n", msg);
+
+		if (err & (ERR_FATAL | ERR_ABORT)) {
+			ret = 0;
+			goto end;
+		}
+	}
+
+	if (l->state < LI_PAUSED) {
+		ret = 0;
+		goto end;
+	}
+
+	if (l->state == LI_PAUSED && l->rx.proto->rx_resume &&
+	    l->rx.proto->rx_resume(&l->rx) <= 0)
+		ret = 0;
+ end:
+	return ret;
+}
+
+
 /* This function tries to temporarily disable a listener, depending on the OS
  * capabilities. Linux unbinds the listen socket after a SHUT_RD, and ignores
  * SHUT_WR. Solaris refuses either shutdown(). OpenBSD ignores SHUT_RD but
@@ -291,37 +406,31 @@ static void disable_listener(struct listener *listener)
  */
 int pause_listener(struct listener *l)
 {
+	struct proxy *px = l->bind_conf->frontend;
 	int ret = 1;
 
-	HA_SPIN_LOCK(LISTENER_LOCK, &l->lock);
-
-	if (l->state <= LI_ZOMBIE)
-		goto end;
+	HA_RWLOCK_WRLOCK(LISTENER_LOCK, &l->lock);
 
 	if ((global.mode & (MODE_DAEMON | MODE_MWORKER)) &&
-	    !(proc_mask(l->bind_conf->bind_proc) & pid_bit))
+	    !(proc_mask(l->rx.settings->bind_proc) & pid_bit))
 		goto end;
 
-	if (l->proto->pause) {
-		/* Returns < 0 in case of failure, 0 if the listener
-		 * was totally stopped, or > 0 if correctly paused.
-		 */
-		int ret = l->proto->pause(l);
+	if (l->state <= LI_PAUSED)
+		goto end;
 
-		if (ret < 0) {
-			ret = 0;
-			goto end;
-		}
-		else if (ret == 0)
-			goto end;
-	}
+	if (l->rx.proto->suspend)
+		ret = l->rx.proto->suspend(l);
 
 	MT_LIST_DEL(&l->wait_queue);
 
-	fd_stop_recv(l->fd);
-	l->state = LI_PAUSED;
+	listener_set_state(l, LI_PAUSED);
+
+	if (px && !px->li_ready) {
+		ha_warning("Paused %s %s.\n", proxy_cap_str(px->cap), px->id);
+		send_log(px, LOG_WARNING, "Paused %s %s.\n", proxy_cap_str(px->cap), px->id);
+	}
   end:
-	HA_SPIN_UNLOCK(LISTENER_LOCK, &l->lock);
+	HA_RWLOCK_WRUNLOCK(LISTENER_LOCK, &l->lock);
 	return ret;
 }
 
@@ -337,9 +446,11 @@ int pause_listener(struct listener *l)
  */
 int resume_listener(struct listener *l)
 {
+	struct proxy *px = l->bind_conf->frontend;
+	int was_paused = px && px->li_paused;
 	int ret = 1;
 
-	HA_SPIN_LOCK(LISTENER_LOCK, &l->lock);
+	HA_RWLOCK_WRLOCK(LISTENER_LOCK, &l->lock);
 
 	/* check that another thread didn't to the job in parallel (e.g. at the
 	 * end of listen_accept() while we'd come from dequeue_all_listeners().
@@ -348,60 +459,31 @@ int resume_listener(struct listener *l)
 		goto end;
 
 	if ((global.mode & (MODE_DAEMON | MODE_MWORKER)) &&
-	    !(proc_mask(l->bind_conf->bind_proc) & pid_bit))
+	    !(proc_mask(l->rx.settings->bind_proc) & pid_bit))
 		goto end;
 
-	if (l->state == LI_ASSIGNED) {
-		char msg[100];
-		int err;
-
-		err = l->proto->bind(l, msg, sizeof(msg));
-		if (err & ERR_ALERT)
-			ha_alert("Resuming listener: %s\n", msg);
-		else if (err & ERR_WARN)
-			ha_warning("Resuming listener: %s\n", msg);
-
-		if (err & (ERR_FATAL | ERR_ABORT)) {
-			ret = 0;
-			goto end;
-		}
-	}
-
-	if (l->state < LI_PAUSED || l->state == LI_ZOMBIE) {
-		ret = 0;
-		goto end;
-	}
-
-	if (l->proto->sock_prot == IPPROTO_TCP &&
-	    l->state == LI_PAUSED &&
-	    listen(l->fd, listener_backlog(l)) != 0) {
-		ret = 0;
-		goto end;
-	}
-
 	if (l->state == LI_READY)
 		goto end;
 
-	MT_LIST_DEL(&l->wait_queue);
+	if (l->rx.proto->resume)
+		ret = l->rx.proto->resume(l);
 
 	if (l->maxconn && l->nbconn >= l->maxconn) {
-		l->state = LI_FULL;
-		goto end;
+		l->rx.proto->disable(l);
+		listener_set_state(l, LI_FULL);
+		goto done;
 	}
 
-	if (!(thread_mask(l->bind_conf->bind_thread) & tid_bit)) {
-		/* we're not allowed to touch this listener's FD, let's requeue
-		 * the listener into one of its owning thread's queue instead.
-		 */
-		int first_thread = my_flsl(thread_mask(l->bind_conf->bind_thread) & all_threads_mask) - 1;
-		work_list_add(&local_listener_queue[first_thread], &l->wait_queue);
-		goto end;
-	}
+	l->rx.proto->enable(l);
+	listener_set_state(l, LI_READY);
 
-	fd_want_recv(l->fd);
-	l->state = LI_READY;
+  done:
+	if (was_paused && !px->li_paused) {
+		ha_warning("Resumed %s %s.\n", proxy_cap_str(px->cap), px->id);
+		send_log(px, LOG_WARNING, "Resumed %s %s.\n", proxy_cap_str(px->cap), px->id);
+	}
   end:
-	HA_SPIN_UNLOCK(LISTENER_LOCK, &l->lock);
+	HA_RWLOCK_WRUNLOCK(LISTENER_LOCK, &l->lock);
 	return ret;
 }
 
@@ -410,15 +492,15 @@ int resume_listener(struct listener *l)
  */
 static void listener_full(struct listener *l)
 {
-	HA_SPIN_LOCK(LISTENER_LOCK, &l->lock);
+	HA_RWLOCK_WRLOCK(LISTENER_LOCK, &l->lock);
 	if (l->state >= LI_READY) {
 		MT_LIST_DEL(&l->wait_queue);
 		if (l->state != LI_FULL) {
-			fd_stop_recv(l->fd);
-			l->state = LI_FULL;
+			l->rx.proto->disable(l);
+			listener_set_state(l, LI_FULL);
 		}
 	}
-	HA_SPIN_UNLOCK(LISTENER_LOCK, &l->lock);
+	HA_RWLOCK_WRUNLOCK(LISTENER_LOCK, &l->lock);
 }
 
 /* Marks a ready listener as limited so that we only try to re-enable it when
@@ -426,48 +508,13 @@ static void listener_full(struct listener *l)
  */
 static void limit_listener(struct listener *l, struct mt_list *list)
 {
-	HA_SPIN_LOCK(LISTENER_LOCK, &l->lock);
+	HA_RWLOCK_WRLOCK(LISTENER_LOCK, &l->lock);
 	if (l->state == LI_READY) {
-		MT_LIST_ADDQ(list, &l->wait_queue);
-		fd_stop_recv(l->fd);
-		l->state = LI_LIMITED;
+		MT_LIST_TRY_ADDQ(list, &l->wait_queue);
+		l->rx.proto->disable(l);
+		listener_set_state(l, LI_LIMITED);
 	}
-	HA_SPIN_UNLOCK(LISTENER_LOCK, &l->lock);
-}
-
-/* This function adds all of the protocol's listener's file descriptors to the
- * polling lists when they are in the LI_LISTEN state. It is intended to be
- * used as a protocol's generic enable_all() primitive, for use after the
- * fork(). It puts the listeners into LI_READY or LI_FULL states depending on
- * their number of connections. It always returns ERR_NONE.
- *
- * Must be called with proto_lock held.
- *
- */
-int enable_all_listeners(struct protocol *proto)
-{
-	struct listener *listener;
-
-	list_for_each_entry(listener, &proto->listeners, proto_list)
-		enable_listener(listener);
-	return ERR_NONE;
-}
-
-/* This function removes all of the protocol's listener's file descriptors from
- * the polling lists when they are in the LI_READY or LI_FULL states. It is
- * intended to be used as a protocol's generic disable_all() primitive. It puts
- * the listeners into LI_LISTEN, and always returns ERR_NONE.
- *
- * Must be called with proto_lock held.
- *
- */
-int disable_all_listeners(struct protocol *proto)
-{
-	struct listener *listener;
-
-	list_for_each_entry(listener, &proto->listeners, proto_list)
-		disable_listener(listener);
-	return ERR_NONE;
+	HA_RWLOCK_WRUNLOCK(LISTENER_LOCK, &l->lock);
 }
 
 /* Dequeues all listeners waiting for a resource the global wait queue */
@@ -496,86 +543,78 @@ void dequeue_proxy_listeners(struct proxy *px)
 	}
 }
 
-/* Must be called with the lock held. Depending on <do_close> value, it does
- * what unbind_listener or unbind_listener_no_close should do.
- */
-void do_unbind_listener(struct listener *listener, int do_close)
-{
-	if (listener->state == LI_READY && fd_updt)
-		fd_stop_recv(listener->fd);
 
+/* default function used to unbind a listener. This is for use by standard
+ * protocols working on top of accepted sockets. The receiver's rx_unbind()
+ * will automatically be used after the listener is disabled if the socket is
+ * still bound. This must be used under the listener's lock.
+ */
+void default_unbind_listener(struct listener *listener)
+{
+	if (listener->state <= LI_ASSIGNED)
+		goto out_close;
+
+	if (listener->rx.fd == -1) {
+		listener_set_state(listener, LI_ASSIGNED);
+		goto out_close;
+	}
+
+	if (listener->state >= LI_READY) {
+		listener->rx.proto->disable(listener);
+		if (listener->rx.flags & RX_F_BOUND)
+			listener_set_state(listener, LI_LISTEN);
+	}
+
+ out_close:
+	if (listener->rx.flags & RX_F_BOUND)
+		listener->rx.proto->rx_unbind(&listener->rx);
+}
+
+/* This function closes the listening socket for the specified listener,
+ * provided that it's already in a listening state. The protocol's unbind()
+ * is called to put the listener into LI_ASSIGNED or LI_LISTEN and handle
+ * the unbinding tasks. The listener enters then the LI_ASSIGNED state if
+ * the receiver is unbound. Must be called with the lock held.
+ */
+void do_unbind_listener(struct listener *listener)
+{
 	MT_LIST_DEL(&listener->wait_queue);
 
-	if (listener->state >= LI_PAUSED) {
-		if (do_close) {
-			fd_delete(listener->fd);
-			listener->fd = -1;
-		}
-		else
-			fd_remove(listener->fd);
-		listener->state = LI_ASSIGNED;
-	}
+	if (listener->rx.proto->unbind)
+		listener->rx.proto->unbind(listener);
+
+	/* we may have to downgrade the listener if the rx was closed */
+	if (!(listener->rx.flags & RX_F_BOUND) && listener->state > LI_ASSIGNED)
+		listener_set_state(listener, LI_ASSIGNED);
 }
 
 /* This function closes the listening socket for the specified listener,
  * provided that it's already in a listening state. The listener enters the
- * LI_ASSIGNED state. This function is intended to be used as a generic
+ * LI_ASSIGNED state, except if the FD is not closed, in which case it may
+ * remain in LI_LISTEN. This function is intended to be used as a generic
  * function for standard protocols.
  */
 void unbind_listener(struct listener *listener)
 {
-	HA_SPIN_LOCK(LISTENER_LOCK, &listener->lock);
-	do_unbind_listener(listener, 1);
-	HA_SPIN_UNLOCK(LISTENER_LOCK, &listener->lock);
-}
-
-/* This function pretends the listener is dead, but keeps the FD opened, so
- * that we can provide it, for conf reloading.
- */
-void unbind_listener_no_close(struct listener *listener)
-{
-	HA_SPIN_LOCK(LISTENER_LOCK, &listener->lock);
-	do_unbind_listener(listener, 0);
-	HA_SPIN_UNLOCK(LISTENER_LOCK, &listener->lock);
-}
-
-/* This function closes all listening sockets bound to the protocol <proto>,
- * and the listeners end in LI_ASSIGNED state if they were higher. It does not
- * detach them from the protocol. It always returns ERR_NONE.
- *
- * Must be called with proto_lock held.
- *
- */
-int unbind_all_listeners(struct protocol *proto)
-{
-	struct listener *listener;
-
-	list_for_each_entry(listener, &proto->listeners, proto_list)
-		unbind_listener(listener);
-	return ERR_NONE;
+	HA_RWLOCK_WRLOCK(LISTENER_LOCK, &listener->lock);
+	do_unbind_listener(listener);
+	HA_RWLOCK_WRUNLOCK(LISTENER_LOCK, &listener->lock);
 }
 
 /* creates one or multiple listeners for bind_conf <bc> on sockaddr <ss> on port
  * range <portl> to <porth>, and possibly attached to fd <fd> (or -1 for auto
- * allocation). The address family is taken from ss->ss_family. The number of
- * jobs and listeners is automatically increased by the number of listeners
- * created. If the <inherited> argument is set to 1, it specifies that the FD
- * was obtained from a parent process.
- * It returns non-zero on success, zero on error with the error message
- * set in <err>.
+ * allocation). The address family is taken from ss->ss_family, and the protocol
+ * passed in <proto> must be usable on this family. The protocol's default iocb
+ * is automatically preset as the receivers' iocb. The number of jobs and
+ * listeners is automatically increased by the number of listeners created. It
+ * returns non-zero on success, zero on error with the error message set in <err>.
  */
 int create_listeners(struct bind_conf *bc, const struct sockaddr_storage *ss,
-                     int portl, int porth, int fd, int inherited, char **err)
+                     int portl, int porth, int fd, struct protocol *proto, char **err)
 {
-	struct protocol *proto = protocol_by_family(ss->ss_family);
 	struct listener *l;
 	int port;
 
-	if (!proto) {
-		memprintf(err, "unsupported protocol family %d", ss->ss_family);
-		return 0;
-	}
-
 	for (port = portl; port <= porth; port++) {
 		l = calloc(1, sizeof(*l));
 		if (!l) {
@@ -586,18 +625,24 @@ int create_listeners(struct bind_conf *bc, const struct sockaddr_storage *ss,
 		LIST_ADDQ(&bc->frontend->conf.listeners, &l->by_fe);
 		LIST_ADDQ(&bc->listeners, &l->by_bind);
 		l->bind_conf = bc;
+		l->rx.settings = &bc->settings;
+		l->rx.owner = l;
+		l->rx.iocb = proto->default_iocb;
+		l->rx.fd = fd;
+
+		memcpy(&l->rx.addr, ss, sizeof(*ss));
+		if (proto->fam->set_port)
+			proto->fam->set_port(&l->rx.addr, port);
 
-		l->fd = fd;
-		memcpy(&l->addr, ss, sizeof(*ss));
 		MT_LIST_INIT(&l->wait_queue);
-		l->state = LI_INIT;
+		listener_set_state(l, LI_INIT);
 
-		proto->add(l, port);
+		proto->add(proto, l);
 
-		if (inherited)
-			l->options |= LI_O_INHERITED;
+		if (fd != -1)
+			l->rx.flags |= RX_F_INHERITED;
 
-		HA_SPIN_INIT(&l->lock);
+		HA_RWLOCK_INIT(&l->lock);
 		_HA_ATOMIC_ADD(&jobs, 1);
 		_HA_ATOMIC_ADD(&listeners, 1);
 	}
@@ -608,23 +653,30 @@ int create_listeners(struct bind_conf *bc, const struct sockaddr_storage *ss,
  * state is automatically updated from LI_ASSIGNED to LI_INIT. The protocol's
  * number of listeners is updated, as well as the global number of listeners
  * and jobs. Note that the listener must have previously been unbound. This
- * is the generic function to use to remove a listener.
- *
- * Will grab the proto_lock.
- *
+ * is a low-level function expected to be called with the proto_lock and the
+ * listener's lock held.
+ */
+void __delete_listener(struct listener *listener)
+{
+	if (listener->state == LI_ASSIGNED) {
+		listener_set_state(listener, LI_INIT);
+		LIST_DEL(&listener->rx.proto_list);
+		listener->rx.proto->nb_receivers--;
+		_HA_ATOMIC_SUB(&jobs, 1);
+		_HA_ATOMIC_SUB(&listeners, 1);
+	}
+}
+
+/* Delete a listener from its protocol's list of listeners (please check
+ * __delete_listener() above). The proto_lock and the listener's lock will
+ * be grabbed in this order.
  */
 void delete_listener(struct listener *listener)
 {
 	HA_SPIN_LOCK(PROTO_LOCK, &proto_lock);
-	HA_SPIN_LOCK(LISTENER_LOCK, &listener->lock);
-	if (listener->state == LI_ASSIGNED) {
-		listener->state = LI_INIT;
-		LIST_DEL(&listener->proto_list);
-		listener->proto->nb_listeners--;
-		_HA_ATOMIC_SUB(&jobs, 1);
-		_HA_ATOMIC_SUB(&listeners, 1);
-	}
-	HA_SPIN_UNLOCK(LISTENER_LOCK, &listener->lock);
+	HA_RWLOCK_WRLOCK(LISTENER_LOCK, &listener->lock);
+	__delete_listener(listener);
+	HA_RWLOCK_WRUNLOCK(LISTENER_LOCK, &listener->lock);
 	HA_SPIN_UNLOCK(PROTO_LOCK, &proto_lock);
 }
 
@@ -653,23 +705,17 @@ int listener_backlog(const struct listener *l)
  * to an accept. It tries to accept as many connections as possible, and for each
  * calls the listener's accept handler (generally the frontend's accept handler).
  */
-void listener_accept(int fd)
+void listener_accept(struct listener *l)
 {
-	struct listener *l = fdtab[fd].owner;
+	struct connection *cli_conn;
 	struct proxy *p;
 	unsigned int max_accept;
 	int next_conn = 0;
 	int next_feconn = 0;
 	int next_actconn = 0;
 	int expire;
-	int cfd;
 	int ret;
-#ifdef USE_ACCEPT4
-	static int accept4_broken;
-#endif
 
-	if (!l)
-		return;
 	p = l->bind_conf->frontend;
 
 	/* if l->maxaccept is -1, then max_accept is UINT_MAX. It is not really
@@ -735,9 +781,8 @@ void listener_accept(int fd)
 	 * shortage, we try again 100ms later in the worst case.
 	 */
 	for (; max_accept; next_conn = next_feconn = next_actconn = 0, max_accept--) {
-		struct sockaddr_storage addr;
-		socklen_t laddr = sizeof(addr);
 		unsigned int count;
+		int status;
 		__decl_thread(unsigned long mask);
 
 		/* pre-increase the number of connections without going too far.
@@ -787,81 +832,40 @@ void listener_accept(int fd)
 			} while (!_HA_ATOMIC_CAS(&actconn, (int *)(&count), next_actconn));
 		}
 
-		/* with sockpair@ we don't want to do an accept */
-		if (unlikely(l->addr.ss_family == AF_CUST_SOCKPAIR)) {
-			if ((cfd = recv_fd_uxst(fd)) != -1)
-				fcntl(cfd, F_SETFL, O_NONBLOCK);
-			/* just like with UNIX sockets, only the family is filled */
-			addr.ss_family = AF_UNIX;
-			laddr = sizeof(addr.ss_family);
-		} else
-
-#ifdef USE_ACCEPT4
-		/* only call accept4() if it's known to be safe, otherwise
-		 * fallback to the legacy accept() + fcntl().
+		/* be careful below, the listener might be shutting down in
+		 * another thread on error and we must not dereference its
+		 * FD without a bit of protection.
 		 */
-		if (unlikely(accept4_broken ||
-			((cfd = accept4(fd, (struct sockaddr *)&addr, &laddr, SOCK_NONBLOCK)) == -1 &&
-			(errno == ENOSYS || errno == EINVAL || errno == EBADF) &&
-			(accept4_broken = 1))))
-#endif
-			if ((cfd = accept(fd, (struct sockaddr *)&addr, &laddr)) != -1)
-				fcntl(cfd, F_SETFL, O_NONBLOCK);
+		cli_conn = NULL;
+		status = CO_AC_PERMERR;
 
-		if (unlikely(cfd == -1)) {
-			switch (errno) {
-			case EAGAIN:
-				if (fdtab[fd].ev & (FD_POLL_HUP|FD_POLL_ERR)) {
-					/* the listening socket might have been disabled in a shared
-					 * process and we're a collateral victim. We'll just pause for
-					 * a while in case it comes back. In the mean time, we need to
-					 * clear this sticky flag.
-					 */
-					_HA_ATOMIC_AND(&fdtab[fd].ev, ~(FD_POLL_HUP|FD_POLL_ERR));
-					goto transient_error;
-				}
-				goto end; /* nothing more to accept */
-			case EINVAL:
-				/* might be trying to accept on a shut fd (eg: soft stop) */
-				goto transient_error;
-			case EINTR:
-			case ECONNABORTED:
+		HA_RWLOCK_RDLOCK(LISTENER_LOCK, &l->lock);
+		if (l->rx.flags & RX_F_BOUND)
+			cli_conn = l->rx.proto->accept_conn(l, &status);
+		HA_RWLOCK_RDUNLOCK(LISTENER_LOCK, &l->lock);
+
+		if (!cli_conn) {
+			switch (status) {
+			case CO_AC_DONE:
+				goto end;
+
+			case CO_AC_RETRY: /* likely a signal */
 				_HA_ATOMIC_SUB(&l->nbconn, 1);
 				if (p)
 					_HA_ATOMIC_SUB(&p->feconn, 1);
 				if (!(l->options & LI_O_UNLIMITED))
 					_HA_ATOMIC_SUB(&actconn, 1);
 				continue;
-			case ENFILE:
-				if (p)
-					send_log(p, LOG_EMERG,
-						 "Proxy %s reached system FD limit (maxsock=%d). Please check system tunables.\n",
-						 p->id, global.maxsock);
-				goto transient_error;
-			case EMFILE:
-				if (p)
-					send_log(p, LOG_EMERG,
-						 "Proxy %s reached process FD limit (maxsock=%d). Please check 'ulimit-n' and restart.\n",
-						 p->id, global.maxsock);
-				goto transient_error;
-			case ENOBUFS:
-			case ENOMEM:
-				if (p)
-					send_log(p, LOG_EMERG,
-						 "Proxy %s reached system memory limit (maxsock=%d). Please check system tunables.\n",
-						 p->id, global.maxsock);
-				goto transient_error;
-			default:
-				/* unexpected result, let's give up and let other tasks run */
+
+			case CO_AC_YIELD:
 				max_accept = 0;
 				goto end;
+
+			default:
+				goto transient_error;
 			}
 		}
 
-		/* we don't want to leak the FD upon reload if it's in the master */
-		if (unlikely(master == 1))
-			fcntl(cfd, F_SETFD, FD_CLOEXEC);
-
 		/* The connection was accepted, it must be counted as such */
 		if (l->counters)
 			HA_ATOMIC_UPDATE_MAX(&l->counters->conn_max, next_conn);
@@ -878,11 +882,12 @@ void listener_accept(int fd)
 
 		_HA_ATOMIC_ADD(&activity[tid].accepted, 1);
 
-		if (unlikely(cfd >= global.maxsock)) {
+		if (unlikely(cli_conn->handle.fd >= global.maxsock)) {
 			send_log(p, LOG_EMERG,
 				 "Proxy %s reached the configured maximum connection limit. Please check the global 'maxconn' value.\n",
 				 p->id);
-			close(cfd);
+			close(cli_conn->handle.fd);
+			conn_free(cli_conn);
 			expire = tick_add(now_ms, 1000); /* try again in 1 second */
 			goto limit_global;
 		}
@@ -896,8 +901,9 @@ void listener_accept(int fd)
 		next_feconn = 0;
 		next_actconn = 0;
 
+
 #if defined(USE_THREAD)
-		mask = thread_mask(l->bind_conf->bind_thread) & all_threads_mask;
+		mask = thread_mask(l->rx.settings->bind_thread) & all_threads_mask;
 		if (atleast2(mask) && (global.tune.options & GTUNE_LISTENER_MQ) && !stopping) {
 			struct accept_queue_ring *ring;
 			unsigned int t, t0, t1, t2;
@@ -998,7 +1004,7 @@ void listener_accept(int fd)
 			 * when processing this loop.
 			 */
 			ring = &accept_queue_rings[t];
-			if (accept_queue_push_mp(ring, cfd, l, &addr, laddr)) {
+			if (accept_queue_push_mp(ring, cli_conn)) {
 				_HA_ATOMIC_ADD(&activity[t].accq_pushed, 1);
 				tasklet_wakeup(ring->tasklet);
 				continue;
@@ -1011,7 +1017,7 @@ void listener_accept(int fd)
 #endif // USE_THREAD
 
 		_HA_ATOMIC_ADD(&l->thr_conn[tid], 1);
-		ret = l->accept(l, cfd, &addr);
+		ret = l->accept(cli_conn);
 		if (unlikely(ret <= 0)) {
 			/* The connection was closed by stream_accept(). Either
 			 * we just have to ignore it (ret == 0) or it's a critical
@@ -1066,31 +1072,20 @@ void listener_accept(int fd)
 		    (!p->fe_sps_lim || freq_ctr_remain(&p->fe_sess_per_sec, p->fe_sps_lim, 0) > 0))
 			dequeue_proxy_listeners(p);
 	}
-
-	/* Now it's getting tricky. The listener was supposed to be in LI_READY
-	 * state but in the mean time we might have changed it to LI_FULL or
-	 * LI_LIMITED, and another thread might also have turned it to
-	 * LI_PAUSED, LI_LISTEN or even LI_INI when stopping a proxy. We must
-	 * be certain to keep the FD enabled when in the READY state but we
-	 * must also stop it for other states that we might have switched to
-	 * while others re-enabled polling.
-	 */
-	HA_SPIN_LOCK(LISTENER_LOCK, &l->lock);
-	if (l->state == LI_READY) {
-		if (max_accept > 0)
-			fd_cant_recv(fd);
-		else
-			fd_done_recv(fd);
-	} else if (l->state > LI_ASSIGNED) {
-		fd_stop_recv(l->fd);
-	}
-	HA_SPIN_UNLOCK(LISTENER_LOCK, &l->lock);
 	return;
 
  transient_error:
 	/* pause the listener for up to 100 ms */
 	expire = tick_add(now_ms, 100);
 
+	/* This may be a shared socket that was paused by another process.
+	 * Let's put it to pause in this case.
+	 */
+	if (l->rx.proto && l->rx.proto->rx_listening(&l->rx) == 0) {
+		pause_listener(l);
+		goto end;
+	}
+
  limit_global:
 	/* (re-)queue the listener to the global queue and set it to expire no
 	 * later than <expire> ahead. The listener turns to LI_LIMITED.
@@ -1135,28 +1130,9 @@ void listener_release(struct listener *l)
 		dequeue_proxy_listeners(fe);
 }
 
-/* resume listeners waiting in the local listener queue. They are still in LI_LIMITED state */
-static struct task *listener_queue_process(struct task *t, void *context, unsigned short state)
-{
-	struct work_list *wl = context;
-	struct listener *l;
-
-	while ((l = MT_LIST_POP(&wl->head, struct listener *, wait_queue))) {
-		/* The listeners are still in the LI_LIMITED state */
-		resume_listener(l);
-	}
-	return t;
-}
-
 /* Initializes the listener queues. Returns 0 on success, otherwise ERR_* flags */
 static int listener_queue_init()
 {
-	local_listener_queue = work_list_create(global.nbthread, listener_queue_process, NULL);
-	if (!local_listener_queue) {
-		ha_alert("Out of memory while initializing listener queues.\n");
-		return ERR_FATAL|ERR_ABORT;
-	}
-
 	global_listener_queue_task = task_new(MAX_THREADS_MASK);
 	if (!global_listener_queue_task) {
 		ha_alert("Out of memory when initializing global listener queue\n");
@@ -1171,7 +1147,6 @@ static int listener_queue_init()
 
 static void listener_queue_deinit()
 {
-	work_list_destroy(local_listener_queue, global.nbthread);
 	task_destroy(global_listener_queue_task);
 	global_listener_queue_task = NULL;
 }
@@ -1184,7 +1159,7 @@ REGISTER_POST_DEINIT(listener_queue_deinit);
  * for global resources when there are enough free resource, or at least once in
  * a while. It is designed to be called as a task.
  */
-static struct task *manage_global_listener_queue(struct task *t, void *context, unsigned short state)
+static struct task *manage_global_listener_queue(struct task *t, void *context, unsigned int state)
 {
 	/* If there are still too many concurrent connections, let's wait for
 	 * some of them to go away. We don't need to re-arm the timer because
@@ -1495,8 +1470,8 @@ static int bind_parse_process(char **args, int cur_arg, struct proxy *px, struct
 		*slash = '/';
 	}
 
-	conf->bind_proc |= proc;
-	conf->bind_thread |= thread;
+	conf->settings.bind_proc |= proc;
+	conf->settings.bind_thread |= thread;
 	return 0;
 }
 
