@@ -129,13 +129,13 @@ void mworker_proc_list_to_env()
 /*
  * unserialize the proc list from the environment
  */
-void mworker_env_to_proc_list()
+int mworker_env_to_proc_list()
 {
 	char *msg, *token = NULL, *s1;
 
 	msg = getenv("HAPROXY_PROCESSES");
 	if (!msg)
-		return;
+		return 0;
 
 	while ((token = strtok_r(msg, "|", &s1))) {
 		struct mworker_proc *child;
@@ -145,6 +145,10 @@ void mworker_env_to_proc_list()
 		msg = NULL;
 
 		child = calloc(1, sizeof(*child));
+		if (!child) {
+			ha_alert("Out of memory while trying to allocate a worker process structure.");
+			return -1;
+		}
 
 		while ((subtoken = strtok_r(token, ";", &s2))) {
 
@@ -191,6 +195,8 @@ void mworker_env_to_proc_list()
 	}
 
 	unsetenv("HAPROXY_PROCESSES");
+
+	return 0;
 }
 
 /* Signal blocking and unblocking */
@@ -441,6 +447,32 @@ void mworker_cleanlisteners()
 	}
 }
 
+/* Upon a configuration loading error some mworker_proc and FDs/server were
+ * assigned but the worker was never forked, we must close the FDs and
+ * remove the server
+ */
+void mworker_cleanup_proc()
+{
+	struct mworker_proc *child, *it;
+
+	list_for_each_entry_safe(child, it, &proc_list, list) {
+
+		if (child->pid == -1) {
+			/* Close the socketpair master side.  We don't need to
+			 * close the worker side, because it's stored in the
+			 * GLOBAL cli listener which was supposed to be in the
+			 * worker and which will be closed in
+			 * mworker_cleanlisteners()
+			 */
+			if (child->ipc_fd[0] > -1)
+				close(child->ipc_fd[0]);
+			LIST_DEL(&child->list);
+			mworker_free_child(child);
+		}
+	}
+}
+
+
 /*  Displays workers and processes  */
 static int cli_io_handler_show_proc(struct appctx *appctx)
 {
