@@ -48,6 +48,70 @@ static int mem_fail_rate = 0;
 static int mem_should_fail(const struct pool_head *);
 #endif
 
+#if defined(HA_HAVE_MALLOC_TRIM)
+static int using_libc_allocator = 0;
+
+/* ask the allocator to trim memory pools.
+ * This must run under thread isolation so that competing threads trying to
+ * allocate or release memory do not prevent the allocator from completing
+ * its job. We just have to be careful as callers might already be isolated
+ * themselves.
+ */
+static void trim_all_pools(void)
+{
+	int isolated = thread_isolated();
+
+	if (!isolated)
+		thread_isolate();
+
+	if (using_libc_allocator)
+		malloc_trim(0);
+
+	if (!isolated)
+		thread_release();
+}
+
+/* check if we're using the same allocator as the one that provides
+ * malloc_trim() and mallinfo(). The principle is that on glibc, both
+ * malloc_trim() and mallinfo() are provided, and using mallinfo() we
+ * can check if malloc() is performed through glibc or any other one
+ * the executable was linked against (e.g. jemalloc).
+ */
+static void detect_allocator(void)
+{
+#ifdef HA_HAVE_MALLINFO2
+	struct mallinfo2 mi1, mi2;
+#else
+	struct mallinfo mi1, mi2;
+#endif
+	void *ptr;
+
+#ifdef HA_HAVE_MALLINFO2
+	mi1 = mallinfo2();
+#else
+	mi1 = mallinfo();
+#endif
+	ptr = DISGUISE(malloc(1));
+#ifdef HA_HAVE_MALLINFO2
+	mi2 = mallinfo2();
+#else
+	mi2 = mallinfo();
+#endif
+	free(DISGUISE(ptr));
+
+	using_libc_allocator = !!memcmp(&mi1, &mi2, sizeof(mi1));
+}
+#else
+
+static void trim_all_pools(void)
+{
+}
+
+static void detect_allocator(void)
+{
+}
+#endif
+
 /* Try to find an existing shared pool with the same characteristics and
  * returns it, otherwise creates this one. NULL is returned if no memory
  * is available for a new creation. Two flags are supported :
@@ -88,7 +152,11 @@ struct pool_head *create_pool(char *name, unsigned int size, unsigned int flags)
 			 * we look for a shareable one or for the next position
 			 * before which we will insert a new one.
 			 */
-			if (flags & entry->flags & MEM_F_SHARED) {
+			if ((flags & entry->flags & MEM_F_SHARED)
+#ifdef DEBUG_DONT_SHARE_POOLS
+			    && strcmp(name, entry->name) == 0
+#endif
+			    ) {
 				/* we can share this one */
 				pool = entry;
 				DPRINTF(stderr, "Sharing %s with %s\n", name, pool->name);
@@ -220,9 +288,7 @@ void pool_flush(struct pool_head *pool)
 /* This function might ask the malloc library to trim its buffers. */
 void pool_gc(struct pool_head *pool_ctx)
 {
-#if defined(HA_HAVE_MALLOC_TRIM)
-	malloc_trim(0);
-#endif
+	trim_all_pools();
 }
 
 #elif defined(CONFIG_HAP_LOCKLESS_POOLS)
@@ -269,11 +335,16 @@ void *__pool_refill_alloc(struct pool_head *pool, unsigned int avail)
 		if (++allocated > avail)
 			break;
 
-		free_list = pool->free_list;
+		free_list = _HA_ATOMIC_LOAD(&pool->free_list);
 		do {
-			*POOL_LINK(pool, ptr) = free_list;
-			__ha_barrier_store();
-		} while (_HA_ATOMIC_CAS(&pool->free_list, &free_list, ptr) == 0);
+			while (unlikely(free_list == POOL_BUSY)) {
+				pl_cpu_relax();
+				free_list = _HA_ATOMIC_LOAD(&pool->free_list);
+			}
+			_HA_ATOMIC_STORE(POOL_LINK(pool, ptr), (void *)free_list);
+			__ha_barrier_atomic_store();
+		} while (!_HA_ATOMIC_CAS(&pool->free_list, &free_list, ptr));
+		__ha_barrier_atomic_store();
 	}
 	__ha_barrier_atomic_store();
 
@@ -298,31 +369,35 @@ void *pool_refill_alloc(struct pool_head *pool, unsigned int avail)
  */
 void pool_flush(struct pool_head *pool)
 {
-	struct pool_free_list cmp, new;
 	void **next, *temp;
 	int removed = 0;
 
 	if (!pool)
 		return;
-	HA_SPIN_LOCK(POOL_LOCK, &pool->lock);
+
+	/* The loop below atomically detaches the head of the free list and
+	 * replaces it with a NULL. Then the list can be released.
+	 */
+	next = pool->free_list;
 	do {
-		cmp.free_list = pool->free_list;
-		cmp.seq = pool->seq;
-		new.free_list = NULL;
-		new.seq = cmp.seq + 1;
-	} while (!_HA_ATOMIC_DWCAS(&pool->free_list, &cmp, &new));
+		while (unlikely(next == POOL_BUSY)) {
+			__ha_cpu_relax();
+			next = _HA_ATOMIC_LOAD(&pool->free_list);
+		}
+		if (next == NULL)
+			return;
+	} while (unlikely((next = _HA_ATOMIC_XCHG(&pool->free_list, POOL_BUSY)) == POOL_BUSY));
+	_HA_ATOMIC_STORE(&pool->free_list, NULL);
 	__ha_barrier_atomic_store();
-	HA_SPIN_UNLOCK(POOL_LOCK, &pool->lock);
-	next = cmp.free_list;
+
 	while (next) {
 		temp = next;
 		next = *POOL_LINK(pool, temp);
 		removed++;
 		pool_free_area(temp, pool->size + POOL_EXTRA);
 	}
-	pool->free_list = next;
 	_HA_ATOMIC_SUB(&pool->allocated, removed);
-	/* here, we should have pool->allocate == pool->used */
+	/* here, we should have pool->allocated == pool->used */
 }
 
 /*
@@ -339,24 +414,19 @@ void pool_gc(struct pool_head *pool_ctx)
 		thread_isolate();
 
 	list_for_each_entry(entry, &pools, list) {
-		while ((int)((volatile int)entry->allocated - (volatile int)entry->used) > (int)entry->minavail) {
-			struct pool_free_list cmp, new;
-
-			cmp.seq = entry->seq;
-			__ha_barrier_load();
-			cmp.free_list = entry->free_list;
-			__ha_barrier_load();
-			if (cmp.free_list == NULL)
-				break;
-			new.free_list = *POOL_LINK(entry, cmp.free_list);
-			new.seq = cmp.seq + 1;
-			if (HA_ATOMIC_DWCAS(&entry->free_list, &cmp, &new) == 0)
-				continue;
-			pool_free_area(cmp.free_list, entry->size + POOL_EXTRA);
-			_HA_ATOMIC_SUB(&entry->allocated, 1);
+		void *temp;
+		//qfprintf(stderr, "Flushing pool %s\n", entry->name);
+		while (entry->free_list &&
+		       (int)(entry->allocated - entry->used) > (int)entry->minavail) {
+			temp = entry->free_list;
+			entry->free_list = *POOL_LINK(entry, temp);
+			entry->allocated--;
+			pool_free_area(temp, entry->size + POOL_EXTRA);
 		}
 	}
 
+	trim_all_pools();
+
 	if (!isolated)
 		thread_release();
 }
@@ -434,21 +504,26 @@ void *pool_refill_alloc(struct pool_head *pool, unsigned int avail)
  */
 void pool_flush(struct pool_head *pool)
 {
-	void *temp;
+	void *temp, **next;
 
 	if (!pool)
 		return;
 
-	while (1) {
-		HA_SPIN_LOCK(POOL_LOCK, &pool->lock);
-		temp = pool->free_list;
-		if (!temp) {
-			HA_SPIN_UNLOCK(POOL_LOCK, &pool->lock);
-			break;
-		}
-		pool->free_list = *POOL_LINK(pool, temp);
+	HA_SPIN_LOCK(POOL_LOCK, &pool->lock);
+	next = pool->free_list;
+	while (next) {
+		temp = next;
+		next = *POOL_LINK(pool, temp);
 		pool->allocated--;
-		HA_SPIN_UNLOCK(POOL_LOCK, &pool->lock);
+	}
+
+	next = pool->free_list;
+	pool->free_list = NULL;
+	HA_SPIN_UNLOCK(POOL_LOCK, &pool->lock);
+
+	while (next) {
+		temp = next;
+		next = *POOL_LINK(pool, temp);
 		pool_free_area(temp, pool->size + POOL_EXTRA);
 	}
 	/* here, we should have pool->allocated == pool->used */
@@ -479,6 +554,8 @@ void pool_gc(struct pool_head *pool_ctx)
 		}
 	}
 
+	trim_all_pools();
+
 	if (!isolated)
 		thread_release();
 }
@@ -520,8 +597,14 @@ void pool_destroy_all()
 {
 	struct pool_head *entry, *back;
 
-	list_for_each_entry_safe(entry, back, &pools, list)
+	list_for_each_entry_safe(entry, back, &pools, list) {
+		/* there's only one occurrence of each pool in the list,
+		 * and we're existing instead of looping on the whole
+		 * list just to decrement users, force it to 1 here.
+		 */
+		entry->users = 1;
 		pool_destroy(entry);
+	}
 }
 
 /* This function dumps memory usage information into the trash buffer. */
@@ -639,6 +722,7 @@ static void init_pools()
 		LIST_INIT(&ha_thread_info[thr].pool_lru_head);
 	}
 #endif
+	detect_allocator();
 }
 
 INITCALL0(STG_PREPARE, init_pools);
