@@ -140,6 +140,10 @@ const char *proxy_mode_str(int mode) {
 		return "http";
 	else if (mode == PR_MODE_CLI)
 		return "cli";
+	else if (mode == PR_MODE_SYSLOG)
+		return "syslog";
+	else if (mode == PR_MODE_PEERS)
+		return "peers";
 	else
 		return "unknown";
 }
@@ -481,6 +485,10 @@ static int proxy_parse_declare(char **args, int section, struct proxy *curpx,
 
 		/* register the capture. */
 		hdr = calloc(1, sizeof(*hdr));
+		if (!hdr) {
+			memprintf(err, "proxy '%s': out of memory while registering a capture", curpx->id);
+			return -1;
+		}
 		hdr->name = NULL; /* not a header capture */
 		hdr->namelen = 0;
 		hdr->len = len;
@@ -1090,7 +1098,7 @@ void proxy_cond_disable(struct proxy *p)
  * called as a task which is woken up upon stopping or when rate limiting must
  * be enforced.
  */
-struct task *manage_proxy(struct task *t, void *context, unsigned short state)
+struct task *manage_proxy(struct task *t, void *context, unsigned int state)
 {
 	struct proxy *p = context;
 	int next = TICK_ETERNITY;
@@ -1194,7 +1202,7 @@ static int proxy_parse_hard_stop_after(char **args, int section_type, struct pro
 	return 0;
 }
 
-struct task *hard_stop(struct task *t, void *context, unsigned short state)
+struct task *hard_stop(struct task *t, void *context, unsigned int state)
 {
 	struct proxy *p;
 	struct stream *s;
